/*
    Project: TCGINA
    Version: V1.16
    File   : fileinfo.c

    Copyright (c) 2005-2007 Author of TCGINA DLL. All rights reserved.
    Public Key Fingerprint: 294B A769 4A0A CC05 DAE6 00DD FF47 8C72 4097 67CE

    Covered by TrueCrypt License 2.2 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include <windows.h>
#include <tchar.h>
#include <tomcrypt.h>
#include <tc.h>
#include "fileinfo.h"

typedef struct {
    WORD wLanguage;
    WORD wCodePage;
} LANG_AND_CODEPAGE, *PLANG_AND_CODEPAGE;

LPCTSTR GetIniFileName(void)
{
    static TCHAR szIniFileName[MAX_PATH] = TEXT("");

    if (*szIniFileName == TEXT('\0')) {
        if (GetModuleFileName(NULL, szIniFileName, MAX_PATH)) {
            LPTSTR pszTemp = _tcsrchr(szIniFileName, TEXT('.'));
            if (pszTemp != NULL) {
                pszTemp[1] = TEXT('\0');
                TC_AppendString(pszTemp, MAX_PATH, TEXT("INI"));
            } else {
                *szIniFileName = TEXT('\0');
            }
        } else {
            *szIniFileName = TEXT('\0');
        }
    }

    return szIniFileName;
}

BOOL GetFileInfo(PFILE_INFO pFileInfo, LPTSTR pszFileName)
{
    HANDLE       hFile;
    DWORD        dwFileVersionInfoSize;
    DWORD        dwResult;

    memset(pFileInfo, '\0', sizeof(*pFileInfo));
    hFile = FindFirstFile(pszFileName, &pFileInfo->fd);
    if (hFile == INVALID_HANDLE_VALUE) {
        TC_CopyString(pFileInfo->fd.cFileName,
          countof(pFileInfo->fd.cFileName), pszFileName);
        return FALSE;
    }
    FindClose(hFile);
    pFileInfo->fExists = TRUE;
    TC_CopyString(pFileInfo->fd.cFileName,
      countof(pFileInfo->fd.cFileName), pszFileName);

    if (pFileInfo->fd.nFileSizeHigh == 0 && pFileInfo->fd.nFileSizeLow != 0 &&
      pFileInfo->fd.nFileSizeLow < 0x100000) {
        void *pData = malloc(pFileInfo->fd.nFileSizeLow);
        if (pData) {
            hFile = CreateFile(pszFileName, GENERIC_READ, FILE_SHARE_READ, NULL,
            OPEN_EXISTING, 0, NULL);
            if (hFile != INVALID_HANDLE_VALUE) {
                DWORD dwFileSize;
                if (ReadFile(hFile, pData, pFileInfo->fd.nFileSizeLow, &dwFileSize, NULL) &&
                  pFileInfo->fd.nFileSizeLow == dwFileSize) {
                    hash_state md;
                    TCHAR      szHash[2*FILE_INFO_HASH_SIZE+1];
                    unsigned   i;
                    sha1_init(&md);
                    sha1_process(&md, pData, dwFileSize);
                    sha1_done(&md, pFileInfo->rgbHashValue);
                    for (i = 0; i < FILE_INFO_HASH_SIZE; i++) {
                        wsprintf(szHash+i*2, TEXT("%02X"),
                          (unsigned)pFileInfo->rgbHashValue[i]);
                    }
                    szHash[i*2] = TEXT('\0');
                    GetPrivateProfileString(TEXT("SHA1"), szHash, TEXT(""),
                      pFileInfo->szInfo, MAX_PATH, GetIniFileName());
                }
                CloseHandle(hFile);
            }
            free(pData);
        }
    }

    hFile = CreateFile(pszFileName, GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0,
      NULL);
    if (hFile != INVALID_HANDLE_VALUE) {
        CloseHandle(hFile);
    } else {
        pFileInfo->fInUse = TRUE;
    }

    dwFileVersionInfoSize = GetFileVersionInfoSize(pszFileName, &dwResult);
    if (dwFileVersionInfoSize) {
        PVOID pBlock = malloc(dwFileVersionInfoSize);
        if (pBlock) {
            if (GetFileVersionInfo(pszFileName, 0, dwFileVersionInfoSize,
              pBlock)) {
                VS_FIXEDFILEINFO   *ffi;
                UINT               uLen;
                PLANG_AND_CODEPAGE pTranslation;
                TCHAR              szValueName[64];
                LPTSTR             pszValueData;
                if (VerQueryValue(pBlock, TEXT("\\"), (LPVOID *)&ffi, &uLen) &&
                  ffi->dwSignature == 0xfeef04bd) {
                    ((DWORD*)&pFileInfo->ullVersion)[0] = ffi->dwFileVersionLS;
                    ((DWORD*)&pFileInfo->ullVersion)[1] = ffi->dwFileVersionMS;
                    pFileInfo->uMajorVersion = ffi->dwFileVersionMS >> 16;
                    pFileInfo->uMinorVersion = ffi->dwFileVersionMS & 0xffff;
                    pFileInfo->uRelease      = ffi->dwFileVersionLS >> 16;
                    pFileInfo->uBuild        = ffi->dwFileVersionLS & 0xffff;
#if 0
                    if (pFileInfo->uBuild == 0 && pFileInfo->uRelease <= 26) {
                        if (pFileInfo->uRelease == 0) {
                            _sntprintf(pFileInfo->szVersion,
                              countof(pFileInfo->szVersion),
                              TEXT("%u.%u"),
                              pFileInfo->uMajorVersion,
                              pFileInfo->uMinorVersion);
                        } else {
                            _sntprintf(pFileInfo->szVersion,
                              countof(pFileInfo->szVersion),
                              TEXT("%u.%u%c"),
                              pFileInfo->uMajorVersion,
                              pFileInfo->uMinorVersion,
                              (TCHAR)(TEXT('a')+(pFileInfo->uRelease-1)));
                        }
                    } else {
#endif
                        _sntprintf(pFileInfo->szVersion,
                          countof(pFileInfo->szVersion),
                          TEXT("%u.%u.%u.%u"),
                          pFileInfo->uMajorVersion,
                          pFileInfo->uMinorVersion,
                          pFileInfo->uRelease,
                          pFileInfo->uBuild);
#if 0
                    }
#endif
                }

                uLen = 0;
                VerQueryValue(pBlock, TEXT("\\VarFileInfo\\Translation"),
                  (LPVOID*)&pTranslation, &uLen);

                if (uLen >= sizeof(LANG_AND_CODEPAGE)) {
                    _sntprintf(szValueName, countof(szValueName),
                      TEXT("\\StringFileInfo\\%04x%04x\\FileVersion"),
                      (unsigned)pTranslation[0].wLanguage,
                      (unsigned)pTranslation[0].wCodePage);
                    if (VerQueryValue(pBlock, szValueName,
                      (LPVOID*)&pszValueData, &uLen)) {
                        TC_CopyString(pFileInfo->szFileVersion,
                          countof(pFileInfo->szFileVersion),
                          pszValueData);
                    }
                    _sntprintf(szValueName, countof(szValueName),
                      TEXT("\\StringFileInfo\\%04x%04x\\ProductName"),
                      (unsigned)pTranslation[0].wLanguage,
                      (unsigned)pTranslation[0].wCodePage);
                    if (VerQueryValue(pBlock, szValueName,
                      (LPVOID*)&pszValueData, &uLen)) {
                        TC_CopyString(pFileInfo->szProductName,
                          countof(pFileInfo->szProductName),
                          pszValueData);
                    }
                    _sntprintf(szValueName, countof(szValueName),
                      TEXT("\\StringFileInfo\\%04x%04x\\ProductVersion"),
                      (unsigned)pTranslation[0].wLanguage,
                      (unsigned)pTranslation[0].wCodePage);
                    if (VerQueryValue(pBlock, szValueName,
                      (LPVOID*)&pszValueData, &uLen)) {
                        TC_CopyString(pFileInfo->szProductVersion,
                          countof(pFileInfo->szProductVersion),
                          pszValueData);
                    }
                    _sntprintf(szValueName, countof(szValueName),
                      TEXT("\\StringFileInfo\\%04x%04x\\LegalCopyright"),
                      (unsigned)pTranslation[0].wLanguage,
                      (unsigned)pTranslation[0].wCodePage);
                    if (VerQueryValue(pBlock, szValueName,
                      (LPVOID*)&pszValueData, &uLen)) {
                        TC_CopyString(pFileInfo->szCopyright,
                          countof(pFileInfo->szCopyright),
                          pszValueData);
                    }
                    _sntprintf(szValueName, countof(szValueName),
                      TEXT("\\StringFileInfo\\%04x%04x\\OriginalFilename"),
                      (unsigned)pTranslation[0].wLanguage,
                      (unsigned)pTranslation[0].wCodePage);
                    if (VerQueryValue(pBlock, szValueName,
                      (LPVOID*)&pszValueData, &uLen)) {
                        TC_CopyString(pFileInfo->szOriginalFileName,
                          countof(pFileInfo->szOriginalFileName),
                          pszValueData);
                        _tcsupr(pFileInfo->szOriginalFileName);
                    }
                    _sntprintf(szValueName, countof(szValueName),
                      TEXT("\\StringFileInfo\\%04x%04x\\CompanyName"),
                      (unsigned)pTranslation[0].wLanguage,
                      (unsigned)pTranslation[0].wCodePage);
                    if (VerQueryValue(pBlock, szValueName,
                      (LPVOID*)&pszValueData, &uLen)) {
                        TC_CopyString(pFileInfo->szCompanyName,
                          countof(pFileInfo->szCompanyName),
                          pszValueData);
                    }
                    _sntprintf(szValueName, countof(szValueName),
                      TEXT("\\StringFileInfo\\%04x%04x\\FileDescription"),
                      (unsigned)pTranslation[0].wLanguage,
                      (unsigned)pTranslation[0].wCodePage);
                    if (VerQueryValue(pBlock, szValueName,
                      (LPVOID*)&pszValueData, &uLen)) {
                        TC_CopyString(pFileInfo->szFileDescription,
                          countof(pFileInfo->szFileDescription),
                          pszValueData);
                    }
                }
            }
            free(pBlock);
        }
    }

    return TRUE;
}

BOOL GetInfoItem(LPCTSTR pszInfo, DWORD dwIndex, LPTSTR pszDest, DWORD nSize)
{
    LPCTSTR pszEnd;
    DWORD   dwItemLen;

    *pszDest = TEXT('\0');
    while (dwIndex--) {
        pszInfo = _tcschr(pszInfo, TEXT(';'));
        if (pszInfo == NULL) return FALSE;
        pszInfo++;
    }
    pszEnd = _tcschr(pszInfo, TEXT(';'));
    if (pszEnd == NULL) {
        pszEnd = pszInfo + _tcslen(pszInfo);
    }
    dwItemLen = (DWORD)(pszEnd - pszInfo);

    if (nSize <= dwItemLen) {
        return FALSE;
    }
    memmove(pszDest, pszInfo, dwItemLen*sizeof(TCHAR));
    pszDest[dwItemLen] = TEXT('\0');

    return TRUE;
}